@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,31 +19,45 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-"""Wrapper class around a file like "/usr/bin/env"
+u"""Wrapper class around a file like "/usr/bin/env"
 
 This class makes certain file operations more convenient and
 associates stat information with filenames
 
 """
 
-import stat, errno, socket, time, re, gzip
+from __future__ import print_function
+from future import standard_library
+standard_library.install_aliases()
+from builtins import str
+from builtins import object
+
+import errno
+import gzip
+import os
+import re
+import shutil
+import socket
+import stat
+import time
 
-from duplicity import tarfile
-from duplicity import file_naming
-from duplicity import globals
-from duplicity import gpg
-from duplicity import util
-from duplicity import librsync
-from duplicity import log #@UnusedImport
-from duplicity import dup_time
 from duplicity import cached_ops
-from duplicity.lazy import * #@UnusedWildImport
+from duplicity import config
+from duplicity import dup_time
+from duplicity import file_naming
+from duplicity import gpg
+from duplicity import librsync
+from duplicity import log
+from duplicity import tarfile
+from duplicity import util
+from duplicity.lazy import *  # pylint: disable=unused-wildcard-import,redefined-builtin
 
 _copy_blocksize = 64 * 1024
 _tmp_path_counter = 1
 
-class StatResult:
-    """Used to emulate the output of os.stat() and related"""
+
+class StatResult(object):
+    u"""Used to emulate the output of os.stat() and related"""
     # st_mode is required by the TarInfo class, but it's unclear how
     # to generate it from file permissions.
     st_mode = 0
@@ -52,171 +66,174 @@ class StatResult:
 class PathException(Exception):
     pass
 
-class ROPath:
-    """Read only Path
+
+class ROPath(object):
+    u"""Read only Path
 
     Objects of this class doesn't represent real files, so they don't
     have a name.  They are required to be indexed though.
 
     """
-    def __init__(self, index, stat = None):
-        """ROPath initializer"""
+    def __init__(self, index, stat=None):  # pylint: disable=unused-argument
+        u"""ROPath initializer"""
         self.opened, self.fileobj = None, None
         self.index = index
         self.stat, self.type = None, None
         self.mode, self.devnums = None, None
 
     def set_from_stat(self):
-        """Set the value of self.type, self.mode from self.stat"""
+        u"""Set the value of self.type, self.mode from self.stat"""
         if not self.stat:
             self.type = None
 
         st_mode = self.stat.st_mode
         if stat.S_ISREG(st_mode):
-            self.type = "reg"
+            self.type = u"reg"
         elif stat.S_ISDIR(st_mode):
-            self.type = "dir"
+            self.type = u"dir"
         elif stat.S_ISLNK(st_mode):
-            self.type = "sym"
+            self.type = u"sym"
         elif stat.S_ISFIFO(st_mode):
-            self.type = "fifo"
+            self.type = u"fifo"
         elif stat.S_ISSOCK(st_mode):
-            raise PathException(util.ufn(self.get_relative_path()) +
+            raise PathException(util.fsdecode(self.get_relative_path()) +
                                 u"is a socket, unsupported by tar")
-            self.type = "sock"
+            self.type = u"sock"  # pylint: disable=unreachable
         elif stat.S_ISCHR(st_mode):
-            self.type = "chr"
+            self.type = u"chr"
         elif stat.S_ISBLK(st_mode):
-            self.type = "blk"
+            self.type = u"blk"
         else:
-            raise PathException("Unknown type")
+            raise PathException(u"Unknown type")
 
         self.mode = stat.S_IMODE(st_mode)
-        if self.type in ("chr", "blk"):
+        if self.type in (u"chr", u"blk"):
             try:
                 self.devnums = (os.major(self.stat.st_rdev),
                                 os.minor(self.stat.st_rdev))
             except:
-                log.Warn(_("Warning: %s invalid devnums (0x%X), treating as (0, 0).")
-                         % (util.ufn(self.get_relative_path()), self.stat.st_rdev))
+                log.Warn(_(u"Warning: %s invalid devnums (0x%X), treating as (0, 0).")
+                         % (util.fsdecode(self.get_relative_path()), self.stat.st_rdev))
                 self.devnums = (0, 0)
 
     def blank(self):
-        """Black out self - set type and stat to None"""
+        u"""Black out self - set type and stat to None"""
         self.type, self.stat = None, None
 
     def exists(self):
-        """True if corresponding file exists"""
+        u"""True if corresponding file exists"""
         return self.type
 
     def isreg(self):
-        """True if self corresponds to regular file"""
-        return self.type == "reg"
+        u"""True if self corresponds to regular file"""
+        return self.type == u"reg"
 
     def isdir(self):
-        """True if self is dir"""
-        return self.type == "dir"
+        u"""True if self is dir"""
+        return self.type == u"dir"
 
     def issym(self):
-        """True if self is sym"""
-        return self.type == "sym"
+        u"""True if self is sym"""
+        return self.type == u"sym"
 
     def isfifo(self):
-        """True if self is fifo"""
-        return self.type == "fifo"
+        u"""True if self is fifo"""
+        return self.type == u"fifo"
 
     def issock(self):
-        """True is self is socket"""
-        return self.type == "sock"
+        u"""True is self is socket"""
+        return self.type == u"sock"
 
     def isdev(self):
-        """True is self is a device file"""
-        return self.type == "chr" or self.type == "blk"
+        u"""True is self is a device file"""
+        return self.type == u"chr" or self.type == u"blk"
 
     def getdevloc(self):
-        """Return device number path resides on"""
+        u"""Return device number path resides on"""
         return self.stat.st_dev
 
     def getsize(self):
-        """Return length in bytes from stat object"""
+        u"""Return length in bytes from stat object"""
         return self.stat.st_size
 
     def getmtime(self):
-        """Return mod time of path in seconds"""
+        u"""Return mod time of path in seconds"""
         return int(self.stat.st_mtime)
 
     def get_relative_path(self):
-        """Return relative path, created from index"""
+        u"""Return relative path, created from index"""
         if self.index:
-            return "/".join(self.index)
+            return b"/".join(self.index)
         else:
-            return "."
+            return b"."
 
     def getperms(self):
-        """Return permissions mode, owner and group"""
+        u"""Return permissions mode, owner and group"""
         s1 = self.stat
-        return '%s:%s %o' % (s1.st_uid, s1.st_gid, self.mode)
+        return u'%s:%s %o' % (s1.st_uid, s1.st_gid, self.mode)
 
     def open(self, mode):
-        """Return fileobj associated with self"""
-        assert mode == "rb" and self.fileobj and not self.opened, \
-               "%s %s %s" % (mode, self.fileobj, self.opened)
+        u"""Return fileobj associated with self"""
+        assert mode == u"rb" and self.fileobj and not self.opened, \
+            u"%s %s %s" % (mode, self.fileobj, self.opened)
         self.opened = 1
         return self.fileobj
 
     def get_data(self):
-        """Return contents of associated fileobj in string"""
-        fin = self.open("rb")
+        u"""Return contents of associated fileobj in string"""
+        fin = self.open(u"rb")
         buf = fin.read()
         assert not fin.close()
         return buf
 
     def setfileobj(self, fileobj):
-        """Set file object returned by open()"""
+        u"""Set file object returned by open()"""
         assert not self.fileobj
         self.fileobj = fileobj
         self.opened = None
 
     def init_from_tarinfo(self, tarinfo):
-        """Set data from tarinfo object (part of tarfile module)"""
+        u"""Set data from tarinfo object (part of tarfile module)"""
         # Set the typepp
-        type = tarinfo.type
+        type = tarinfo.type  # pylint: disable=redefined-builtin
         if type == tarfile.REGTYPE or type == tarfile.AREGTYPE:
-            self.type = "reg"
+            self.type = u"reg"
         elif type == tarfile.LNKTYPE:
-            raise PathException("Hard links not supported yet")
+            raise PathException(u"Hard links not supported yet")
         elif type == tarfile.SYMTYPE:
-            self.type = "sym"
+            self.type = u"sym"
             self.symtext = tarinfo.linkname
+            if isinstance(self.symtext, u"".__class__):
+                self.symtext = util.fsencode(self.symtext)
         elif type == tarfile.CHRTYPE:
-            self.type = "chr"
+            self.type = u"chr"
             self.devnums = (tarinfo.devmajor, tarinfo.devminor)
         elif type == tarfile.BLKTYPE:
-            self.type = "blk"
+            self.type = u"blk"
             self.devnums = (tarinfo.devmajor, tarinfo.devminor)
         elif type == tarfile.DIRTYPE:
-            self.type = "dir"
+            self.type = u"dir"
         elif type == tarfile.FIFOTYPE:
-            self.type = "fifo"
+            self.type = u"fifo"
         else:
-            raise PathException("Unknown tarinfo type %s" % (type,))
+            raise PathException(u"Unknown tarinfo type %s" % (type,))
 
         self.mode = tarinfo.mode
         self.stat = StatResult()
 
-        """ Set user and group id
+        u""" Set user and group id
         use numeric id if name lookup fails
         OR
         --numeric-owner is set
         """
         try:
-            if globals.numeric_owner:
+            if config.numeric_owner:
                 raise KeyError
             self.stat.st_uid = cached_ops.getpwnam(tarinfo.uname)[2]
         except KeyError:
             self.stat.st_uid = tarinfo.uid
         try:
-            if globals.numeric_owner:
+            if config.numeric_owner:
                 raise KeyError
             self.stat.st_gid = cached_ops.getgrnam(tarinfo.gname)[2]
         except KeyError:
@@ -224,13 +241,13 @@ class ROPath:
 
         self.stat.st_mtime = int(tarinfo.mtime)
         if self.stat.st_mtime < 0:
-            log.Warn(_("Warning: %s has negative mtime, treating as 0.")
-                     % (util.ufn(tarinfo.name)))
+            log.Warn(_(u"Warning: %s has negative mtime, treating as 0.")
+                     % (tarinfo.uc_name))
             self.stat.st_mtime = 0
         self.stat.st_size = tarinfo.size
 
     def get_ropath(self):
-        """Return ropath copy of self"""
+        u"""Return ropath copy of self"""
         new_ropath = ROPath(self.index, self.stat)
         new_ropath.type, new_ropath.mode = self.type, self.mode
         if self.issym():
@@ -242,7 +259,7 @@ class ROPath:
         return new_ropath
 
     def get_tarinfo(self):
-        """Generate a tarfile.TarInfo object based on self
+        u"""Generate a tarfile.TarInfo object based on self
 
         Doesn't set size based on stat, because we may want to replace
         data wiht other stream.  Size should be set separately by
@@ -251,11 +268,11 @@ class ROPath:
         """
         ti = tarfile.TarInfo()
         if self.index:
-            ti.name = "/".join(self.index)
+            ti.name = util.fsdecode(b"/".join(self.index))
         else:
-            ti.name = "."
+            ti.name = u"."
         if self.isdir():
-            ti.name += "/" # tar dir naming convention
+            ti.name += u"/"  # tar dir naming convention
 
         ti.size = 0
         if self.type:
@@ -271,20 +288,22 @@ class ROPath:
             elif self.issym():
                 ti.type = tarfile.SYMTYPE
                 ti.linkname = self.symtext
+                if isinstance(ti.linkname, bytes):
+                    ti.linkname = util.fsdecode(ti.linkname)
             elif self.isdev():
-                if self.type == "chr":
+                if self.type == u"chr":
                     ti.type = tarfile.CHRTYPE
                 else:
                     ti.type = tarfile.BLKTYPE
                 ti.devmajor, ti.devminor = self.devnums
             else:
-                raise PathException("Unrecognized type " + str(self.type))
+                raise PathException(u"Unrecognized type " + str(self.type))
 
             ti.mode = self.mode
             ti.uid, ti.gid = self.stat.st_uid, self.stat.st_gid
             if self.stat.st_mtime < 0:
-                log.Warn(_("Warning: %s has negative mtime, treating as 0.")
-                         % (util.ufn(self.get_relative_path())))
+                log.Warn(_(u"Warning: %s has negative mtime, treating as 0.")
+                         % (util.fsdecode(self.get_relative_path())))
                 ti.mtime = 0
             else:
                 ti.mtime = int(self.stat.st_mtime)
@@ -292,14 +311,14 @@ class ROPath:
             try:
                 ti.uname = cached_ops.getpwuid(ti.uid)[0]
             except KeyError:
-                ti.uname = ''
+                ti.uname = u''
             try:
                 ti.gname = cached_ops.getgrgid(ti.gid)[0]
             except KeyError:
-                ti.gname = ''
+                ti.gname = u''
 
             if ti.type in (tarfile.CHRTYPE, tarfile.BLKTYPE):
-                if hasattr(os, "major") and hasattr(os, "minor"):
+                if hasattr(os, u"major") and hasattr(os, u"minor"):
                     ti.devmajor, ti.devminor = self.devnums
         else:
             # Currently we depend on an uninitiliazed tarinfo file to
@@ -309,9 +328,9 @@ class ROPath:
         return ti
 
     def __eq__(self, other):
-        """Used to compare two ROPaths.  Doesn't look at fileobjs"""
+        u"""Used to compare two ROPaths.  Doesn't look at fileobjs"""
         if not self.type and not other.type:
-            return 1 # neither exists
+            return 1  # neither exists
         if not self.stat and other.stat or not other.stat and self.stat:
             return 0
         if self.type != other.type:
@@ -336,79 +355,83 @@ class ROPath:
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def compare_verbose(self, other, include_data = 0):
-        """Compare ROPaths like __eq__, but log reason if different
+    def compare_verbose(self, other, include_data=0):
+        u"""Compare ROPaths like __eq__, but log reason if different
 
         This is placed in a separate function from __eq__ because
         __eq__ should be very time sensitive, and logging statements
         would slow it down.  Used when verifying.
 
-        If include_data is true, also read all the data of regular
-        files and see if they differ.
+        Only run if include_data is true.
 
         """
         def log_diff(log_string):
-            log_str = _("Difference found:") + u" " + log_string
-            log.Notice(log_str % (util.ufn(self.get_relative_path())))
+            log_str = _(u"Difference found:") + u" " + log_string
+            log.Notice(log_str % (util.fsdecode(self.get_relative_path())))
+
+        if include_data is False:
+            return True
 
         if not self.type and not other.type:
             return 1
         if not self.stat and other.stat:
-            log_diff(_("New file %s"))
+            log_diff(_(u"New file %s"))
             return 0
         if not other.stat and self.stat:
-            log_diff(_("File %s is missing"))
+            log_diff(_(u"File %s is missing"))
             return 0
         if self.type != other.type:
-            log_diff(_("File %%s has type %s, expected %s") %
+            log_diff(_(u"File %%s has type %s, expected %s") %
                      (other.type, self.type))
             return 0
 
         if self.isreg() or self.isdir() or self.isfifo():
             if not self.perms_equal(other):
-                log_diff(_("File %%s has permissions %s, expected %s") %
+                log_diff(_(u"File %%s has permissions %s, expected %s") %
                          (other.getperms(), self.getperms()))
                 return 0
             if ((int(self.stat.st_mtime) != int(other.stat.st_mtime)) and
-                (self.stat.st_mtime > 0 or other.stat.st_mtime > 0)):
-                log_diff(_("File %%s has mtime %s, expected %s") %
+                    (self.stat.st_mtime > 0 or other.stat.st_mtime > 0)):
+                log_diff(_(u"File %%s has mtime %s, expected %s") %
                          (dup_time.timetopretty(int(other.stat.st_mtime)),
                           dup_time.timetopretty(int(self.stat.st_mtime))))
                 return 0
-            if self.isreg() and include_data:
+            if self.isreg():
                 if self.compare_data(other):
                     return 1
                 else:
-                    log_diff(_("Data for file %s is different"))
+                    log_diff(_(u"Data for file %s is different"))
                     return 0
             else:
                 return 1
         elif self.issym():
-            if self.symtext == other.symtext:
+            if self.symtext == other.symtext or self.symtext + u"/" == other.symtext:
                 return 1
             else:
-                log_diff(_("Symlink %%s points to %s, expected %s") %
+                log_diff(_(u"Symlink %%s points to %s, expected %s") %
                          (other.symtext, self.symtext))
                 return 0
         elif self.isdev():
             if not self.perms_equal(other):
-                log_diff(_("File %%s has permissions %s, expected %s") %
+                log_diff(_(u"File %%s has permissions %s, expected %s") %
                          (other.getperms(), self.getperms()))
                 return 0
             if self.devnums != other.devnums:
-                log_diff(_("Device file %%s has numbers %s, expected %s")
+                log_diff(_(u"Device file %%s has numbers %s, expected %s")
                          % (other.devnums, self.devnums))
                 return 0
             return 1
         assert 0
 
     def compare_data(self, other):
-        """Compare data from two regular files, return true if same"""
-        f1 = self.open("rb")
-        f2 = other.open("rb")
+        u"""Compare data from two regular files, return true if same"""
+        f1 = self.open(u"rb")
+        f2 = other.open(u"rb")
+
         def close():
             assert not f1.close()
             assert not f2.close()
+
         while 1:
             buf1 = f1.read(_copy_blocksize)
             buf2 = f2.read(_copy_blocksize)
@@ -420,38 +443,39 @@ class ROPath:
                 return 1
 
     def perms_equal(self, other):
-        """True if self and other have same permissions and ownership"""
+        u"""True if self and other have same permissions and ownership"""
         s1, s2 = self.stat, other.stat
         return (self.mode == other.mode and
                 s1.st_gid == s2.st_gid and s1.st_uid == s2.st_uid)
 
     def copy(self, other):
-        """Copy self to other.  Also copies data.  Other must be Path"""
+        u"""Copy self to other.  Also copies data.  Other must be Path"""
         if self.isreg():
-            other.writefileobj(self.open("rb"))
+            other.writefileobj(self.open(u"rb"))
         elif self.isdir():
             os.mkdir(other.name)
         elif self.issym():
             os.symlink(self.symtext, other.name)
             os.lchown(other.name, self.stat.st_uid, self.stat.st_gid)
             other.setdata()
-            return # no need to copy symlink attributes
+            return  # no need to copy symlink attributes
         elif self.isfifo():
             os.mkfifo(other.name)
         elif self.issock():
             socket.socket(socket.AF_UNIX).bind(other.name)
         elif self.isdev():
-            if self.type == "chr":
-                devtype = "c"
+            if self.type == u"chr":
+                devtype = u"c"
             else:
-                devtype = "b"
+                devtype = u"b"
             other.makedev(devtype, *self.devnums)
         self.copy_attribs(other)
 
     def copy_attribs(self, other):
-        """Only copy attributes from self to other"""
+        u"""Only copy attributes from self to other"""
         if isinstance(other, Path):
-            util.maybe_ignore_errors(lambda: os.chown(other.name, self.stat.st_uid, self.stat.st_gid))
+            if self.stat is not None:
+                util.maybe_ignore_errors(lambda: os.chown(other.name, self.stat.st_uid, self.stat.st_gid))
             util.maybe_ignore_errors(lambda: os.chmod(other.name, self.mode))
             util.maybe_ignore_errors(lambda: os.utime(other.name, (time.time(), self.stat.st_mtime)))
             other.setdata()
@@ -464,51 +488,68 @@ class ROPath:
             other.stat = stat
             other.mode = self.mode
 
-    def __unicode__(self):
-        """Return string representation"""
+    def __str__(self):
+        u"""Return string representation"""
         return u"(%s %s)" % (util.uindex(self.index), self.type)
 
 
 class Path(ROPath):
-    """
+    u"""
     Path class - wrapper around ordinary local files
 
     Besides caching stat() results, this class organizes various file
     code.
     """
-    regex_chars_to_quote = re.compile("[\\\\\\\"\\$`]")
+    regex_chars_to_quote = re.compile(u"[\\\\\\\"\\$`]")
 
     def rename_index(self, index):
-        if not globals.rename or not index:
-            return index # early exit
+        if not config.rename or not index:
+            return index  # early exit
         path = os.path.normcase(os.path.join(*index))
         tail = []
-        while path and path not in globals.rename:
+        while path and path not in config.rename:
             path, extra = os.path.split(path)
             tail.insert(0, extra)
         if path:
-            return globals.rename[path].split(os.sep) + tail
+            return config.rename[path].split(os.sep) + tail
         else:
-            return index # no rename found
+            return index  # no rename found
 
-    def __init__(self, base, index = ()):
-        """Path initializer"""
+    def __init__(self, base, index=()):
+        u"""Path initializer"""
         # self.opened should be true if the file has been opened, and
         # self.fileobj can override returned fileobj
         self.opened, self.fileobj = None, None
+        if isinstance(base, str):
+            # For now (Python 2), it is helpful to know that all paths
+            # are starting with bytes -- see note above util.fsencode definition
+            base = util.fsencode(base)
         self.base = base
+
+        # Create self.index, which is the path as a tuple
         self.index = self.rename_index(index)
+
         self.name = os.path.join(base, *self.index)
+
+        # We converted any unicode base to filesystem encoding, so self.name should
+        # be in filesystem encoding already and does not need to change
+        self.uc_name = util.fsdecode(self.name)
+
         self.setdata()
 
     def setdata(self):
-        """Refresh stat cache"""
+        u"""Refresh stat cache"""
         try:
-            self.stat = os.lstat(self.name)
-        except OSError, e:
-            err_string = errno.errorcode[e[0]]
-            if err_string in ["ENOENT", "ENOTDIR", "ELOOP", "ENOTCONN"]:
-                self.stat, self.type = None, None # file doesn't exist
+            # We may be asked to look at the target of symlinks rather than
+            # the link itself.
+            if config.copy_links:
+                self.stat = os.stat(self.name)
+            else:
+                self.stat = os.lstat(self.name)
+        except OSError as e:
+            err_string = errno.errorcode[e.errno]
+            if err_string in [u"ENOENT", u"ENOTDIR", u"ELOOP", u"ENOTCONN", u"ENODEV"]:
+                self.stat, self.type = None, None  # file doesn't exist
                 self.mode = None
             else:
                 raise
@@ -518,23 +559,25 @@ class Path(ROPath):
                 self.symtext = os.readlink(self.name)
 
     def append(self, ext):
-        """Return new Path with ext added to index"""
+        u"""Return new Path with ext added to index"""
+        if isinstance(ext, u"".__class__):
+            ext = util.fsencode(ext)
         return self.__class__(self.base, self.index + (ext,))
 
     def new_index(self, index):
-        """Return new Path with index index"""
+        u"""Return new Path with index index"""
         return self.__class__(self.base, index)
 
     def listdir(self):
-        """Return list generated by os.listdir"""
+        u"""Return list generated by os.listdir"""
         return os.listdir(self.name)
 
     def isemptydir(self):
-        """Return true if path is a directory and is empty"""
+        u"""Return true if path is a directory and is empty"""
         return self.isdir() and not self.listdir()
 
-    def open(self, mode = "rb"):
-        """
+    def open(self, mode=u"rb"):
+        u"""
         Return fileobj associated with self
 
         Usually this is just the file data on disk, but can be
@@ -547,26 +590,26 @@ class Path(ROPath):
             result = open(self.name, mode)
         return result
 
-    def makedev(self, type, major, minor):
-        """Make a device file with specified type, major/minor nums"""
-        cmdlist = ['mknod', self.name, type, str(major), str(minor)]
-        if os.spawnvp(os.P_WAIT, 'mknod', cmdlist) != 0:
-            raise PathException("Error running %s" % cmdlist)
+    def makedev(self, type, major, minor):  # pylint: disable=redefined-builtin
+        u"""Make a device file with specified type, major/minor nums"""
+        cmdlist = [u'mknod', self.name, type, str(major), str(minor)]
+        if os.spawnvp(os.P_WAIT, u'mknod', cmdlist) != 0:
+            raise PathException(u"Error running %s" % cmdlist)
         self.setdata()
 
     def mkdir(self):
-        """Make directory(s) at specified path"""
-        log.Info(_("Making directory %s") % util.ufn(self.name))
+        u"""Make directory(s) at specified path"""
+        log.Info(_(u"Making directory %s") % self.uc_name)
         try:
             os.makedirs(self.name)
         except OSError:
-            if (not globals.force):
-                raise PathException("Error creating directory %s" % util.ufn(self.name), 7)
+            if (not config.force):
+                raise PathException(u"Error creating directory %s" % self.uc_name, 7)
         self.setdata()
 
     def delete(self):
-        """Remove this file"""
-        log.Info(_("Deleting %s") % util.ufn(self.name))
+        u"""Remove this file"""
+        log.Info(_(u"Deleting %s") % self.uc_name)
         if self.isdir():
             util.ignore_missing(os.rmdir, self.name)
         else:
@@ -574,15 +617,15 @@ class Path(ROPath):
         self.setdata()
 
     def touch(self):
-        """Open the file, write 0 bytes, close"""
-        log.Info(_("Touching %s") % util.ufn(self.name))
-        fp = self.open("wb")
+        u"""Open the file, write 0 bytes, close"""
+        log.Info(_(u"Touching %s") % self.uc_name)
+        fp = self.open(u"wb")
         fp.close()
 
     def deltree(self):
-        """Remove self by recursively deleting files under it"""
-        from duplicity import selection # todo: avoid circ. dep. issue
-        log.Info(_("Deleting tree %s") % util.ufn(self.name))
+        u"""Remove self by recursively deleting files under it"""
+        from duplicity import selection  # todo: avoid circ. dep. issue
+        log.Info(_(u"Deleting tree %s") % self.uc_name)
         itr = IterTreeReducer(PathDeleter, [])
         for path in selection.Select(self).set_iter():
             itr(path.index, path)
@@ -590,96 +633,100 @@ class Path(ROPath):
         self.setdata()
 
     def get_parent_dir(self):
-        """Return directory that self is in"""
+        u"""Return directory that self is in"""
         if self.index:
             return Path(self.base, self.index[:-1])
         else:
-            components = self.base.split("/")
+            components = self.base.split(b"/")
             if len(components) == 2 and not components[0]:
-                return Path("/") # already in root directory
+                return Path(b"/")  # already in root directory
             else:
-                return Path("/".join(components[:-1]))
+                return Path(b"/".join(components[:-1]))
 
     def writefileobj(self, fin):
-        """Copy file object fin to self.  Close both when done."""
-        fout = self.open("wb")
+        u"""Copy file object fin to self.  Close both when done."""
+        fout = self.open(u"wb")
         while 1:
             buf = fin.read(_copy_blocksize)
             if not buf:
                 break
             fout.write(buf)
         if fin.close() or fout.close():
-            raise PathException("Error closing file object")
+            raise PathException(u"Error closing file object")
         self.setdata()
 
     def rename(self, new_path):
-        """Rename file at current path to new_path."""
-        os.rename(self.name, new_path.name)
+        u"""Rename file at current path to new_path."""
+        shutil.move(self.name, new_path.name)
         self.setdata()
         new_path.setdata()
 
     def move(self, new_path):
-        """Like rename but destination may be on different file system"""
+        u"""Like rename but destination may be on different file system"""
         self.copy(new_path)
         self.delete()
 
     def chmod(self, mode):
-        """Change permissions of the path"""
+        u"""Change permissions of the path"""
         os.chmod(self.name, mode)
         self.setdata()
 
     def patch_with_attribs(self, diff_ropath):
-        """Patch self with diff and then copy attributes over"""
+        u"""Patch self with diff and then copy attributes over"""
         assert self.isreg() and diff_ropath.isreg()
         temp_path = self.get_temp_in_same_dir()
-        patch_fileobj = librsync.PatchedFile(self.open("rb"),
-                                             diff_ropath.open("rb"))
+        fbase = self.open(u"rb")
+        fdiff = diff_ropath.open(u"rb")
+        patch_fileobj = librsync.PatchedFile(fbase, fdiff)
         temp_path.writefileobj(patch_fileobj)
+        assert not fbase.close()
+        assert not fdiff.close()
         diff_ropath.copy_attribs(temp_path)
         temp_path.rename(self)
 
     def get_temp_in_same_dir(self):
-        """Return temp non existent path in same directory as self"""
+        u"""Return temp non existent path in same directory as self"""
         global _tmp_path_counter
         parent_dir = self.get_parent_dir()
         while 1:
-            temp_path = parent_dir.append("duplicity_temp." +
+            temp_path = parent_dir.append(u"duplicity_temp." +
                                           str(_tmp_path_counter))
             if not temp_path.type:
                 return temp_path
             _tmp_path_counter += 1
             assert _tmp_path_counter < 10000, \
-                   u"Warning too many temp files created for " + util.ufn(self.name)
+                u"Warning too many temp files created for " + self.uc_name
 
-    def compare_recursive(self, other, verbose = None):
-        """Compare self to other Path, descending down directories"""
-        from duplicity import selection # todo: avoid circ. dep. issue
+    def compare_recursive(self, other, verbose=None):
+        u"""Compare self to other Path, descending down directories"""
+        from duplicity import selection  # todo: avoid circ. dep. issue
         selfsel = selection.Select(self).set_iter()
         othersel = selection.Select(other).set_iter()
         return Iter.equal(selfsel, othersel, verbose)
 
     def __repr__(self):
-        """Return string representation"""
-        return "(%s %s %s)" % (self.index, self.name, self.type)
+        u"""Return string representation"""
+        return u"(%s %s %s)" % (self.index, self.name, self.type)
 
-    def quote(self, s = None):
-        """
+    def quote(self, s=None):
+        u"""
         Return quoted version of s (defaults to self.name)
 
         The output is meant to be interpreted with shells, so can be
         used with os.system.
         """
         if not s:
-            s = self.name
-        return '"%s"' % self.regex_chars_to_quote.sub(lambda m: "\\"+m.group(0), s)
+            s = self.uc_name
+        return u'"%s"' % self.regex_chars_to_quote.sub(lambda m: u"\\" + m.group(0), s)
 
     def unquote(self, s):
-        """Return unquoted version of string s, as quoted by above quote()"""
-        assert s[0] == s[-1] == "\"" # string must be quoted by above
-        result = ""; i = 1
-        while i < len(s)-1:
-            if s[i] == "\\":
-                result += s[i+1]
+        u"""Return unquoted version of string s, as quoted by above quote()"""
+        assert s[0] == s[-1] == u"\""  # string must be quoted by above
+        result = u""
+        i = 1
+        while i < len(s) - 1:
+            if s[i] == u"\\":
+                result += s[i + 1]
                 i += 2
             else:
                 result += s[i]
@@ -687,38 +734,37 @@ class Path(ROPath):
         return result
 
     def get_filename(self):
-        """Return filename of last component"""
-        components = self.name.split("/")
+        u"""Return filename of last component"""
+        components = self.name.split(b"/")
         assert components and components[-1]
         return components[-1]
 
     def get_canonical(self):
-        """
+        u"""
         Return string of canonical version of path
 
         Remove ".", and trailing slashes where possible.  Note that
         it's harder to remove "..", as "foo/bar/.." is not necessarily
         "foo", so we can't use path.normpath()
         """
-        newpath = "/".join(filter(lambda x: x and x != ".",
-                                  self.name.split("/")))
-        if self.name[0] == "/":
-            return "/" + newpath
+        newpath = b"/".join([x for x in self.name.split(b"/") if x and x != b"."])
+        if self.uc_name[0] == u"/":
+            return b"/" + newpath
         elif newpath:
             return newpath
         else:
-            return "."
+            return b"."
 
 
 class DupPath(Path):
-    """
+    u"""
     Represent duplicity data files
 
     Based on the file name, files that are compressed or encrypted
     will have different open() methods.
     """
-    def __init__(self, base, index = (), parseresults = None):
-        """
+    def __init__(self, base, index=(), parseresults=None):
+        u"""
         DupPath initializer
 
         The actual filename (no directory) must be the single element
@@ -730,16 +776,16 @@ class DupPath(Path):
         else:
             assert len(index) == 1
             self.pr = file_naming.parse(index[0])
-            assert self.pr, "must be a recognizable duplicity file"
+            assert self.pr, u"must be a recognizable duplicity file"
 
         Path.__init__(self, base, index)
 
-    def filtered_open(self, mode = "rb", gpg_profile = None):
-        """
+    def filtered_open(self, mode=u"rb", gpg_profile=None):
+        u"""
         Return fileobj with appropriate encryption/compression
 
         If encryption is specified but no gpg_profile, use
-        globals.default_profile.
+        config.default_profile.
         """
         assert not self.opened and not self.fileobj
         assert not (self.pr.encrypted and self.pr.compressed)
@@ -750,22 +796,25 @@ class DupPath(Path):
             return gzip.GzipFile(self.name, mode)
         elif self.pr.encrypted:
             if not gpg_profile:
-                gpg_profile = globals.gpg_profile
-            if mode == "rb":
+                gpg_profile = config.gpg_profile
+            if mode == u"rb":
                 return gpg.GPGFile(False, self, gpg_profile)
-            elif mode == "wb":
+            elif mode == u"wb":
                 return gpg.GPGFile(True, self, gpg_profile)
         else:
             return self.open(mode)
 
 
 class PathDeleter(ITRBranch):
-    """Delete a directory.  Called by Path.deltree"""
-    def start_process(self, index, path):
+    u"""Delete a directory.  Called by Path.deltree"""
+    def start_process(self, index, path):  # pylint: disable=unused-argument
         self.path = path
+
     def end_process(self):
         self.path.delete()
-    def can_fast_process(self, index, path):
+
+    def can_fast_process(self, index, path):  # pylint: disable=unused-argument
         return not path.isdir()
-    def fast_process(self, index, path):
+
+    def fast_process(self, index, path):  # pylint: disable=unused-argument
         path.delete()
